"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useOperation = void 0;
const tslib_1 = require("tslib");
const react_1 = require("react");
const retil_support_1 = require("retil-support");
function useOperation(operation, deps) {
    // The types don't allow an `undefined` value to be passed in, but it works.
    //@ts-ignore
    // eslint-disable-next-line react-hooks/exhaustive-deps
    const memoizedOperation = react_1.useCallback(operation, deps);
    const [state, setState] = react_1.useState([false]);
    const abortControllerRef = react_1.useRef(null);
    react_1.useEffect(() => () => {
        if (abortControllerRef.current) {
            abortControllerRef.current.abort();
            abortControllerRef.current = null;
        }
    }, []);
    const trigger = react_1.useCallback((data) => tslib_1.__awaiter(this, void 0, void 0, function* () {
        if (abortControllerRef.current) {
            abortControllerRef.current.abort();
        }
        const abortController = new AbortController();
        const signal = abortController.signal;
        abortControllerRef.current = abortController;
        const resultPromise = memoizedOperation(data, signal);
        if (resultPromise && retil_support_1.isPromiseLike(resultPromise)) {
            const instantProbe = Symbol();
            const probeResult = yield Promise.race([
                resultPromise,
                new Promise((resolve) => setTimeout(() => resolve(instantProbe))),
            ]);
            if (probeResult !== instantProbe) {
                // If the result promise resolves before a newly created promise, then
                // let's skip the pending state and immediately set the result on state.
                if (abortControllerRef.current === abortController) {
                    setState([false, probeResult]);
                }
                return probeResult;
            }
            else {
                if (abortControllerRef.current === abortController) {
                    setState([true]);
                }
                const result = yield resultPromise;
                if (abortControllerRef.current === abortController) {
                    abortControllerRef.current = null;
                    setState([false, result]);
                }
                return result;
            }
        }
        else {
            setState([false, resultPromise]);
            return resultPromise;
        }
    }), [memoizedOperation]);
    return [trigger].concat(state);
}
exports.useOperation = useOperation;
