import { __awaiter } from "tslib";
import { useCallback, useEffect, useRef, useState } from 'react';
import { isPromiseLike } from 'retil-support';
export function useOperation(operation, deps) {
    // The types don't allow an `undefined` value to be passed in, but it works.
    //@ts-ignore
    // eslint-disable-next-line react-hooks/exhaustive-deps
    const memoizedOperation = useCallback(operation, deps);
    const [state, setState] = useState([false]);
    const abortControllerRef = useRef(null);
    useEffect(() => () => {
        if (abortControllerRef.current) {
            abortControllerRef.current.abort();
            abortControllerRef.current = null;
        }
    }, []);
    const trigger = useCallback((data) => __awaiter(this, void 0, void 0, function* () {
        if (abortControllerRef.current) {
            abortControllerRef.current.abort();
        }
        const abortController = new AbortController();
        const signal = abortController.signal;
        abortControllerRef.current = abortController;
        const resultPromise = memoizedOperation(data, signal);
        if (resultPromise && isPromiseLike(resultPromise)) {
            const instantProbe = Symbol();
            const probeResult = yield Promise.race([
                resultPromise,
                new Promise((resolve) => setTimeout(() => resolve(instantProbe))),
            ]);
            if (probeResult !== instantProbe) {
                // If the result promise resolves before a newly created promise, then
                // let's skip the pending state and immediately set the result on state.
                if (abortControllerRef.current === abortController) {
                    setState([false, probeResult]);
                }
                return probeResult;
            }
            else {
                if (abortControllerRef.current === abortController) {
                    setState([true]);
                }
                const result = yield resultPromise;
                if (abortControllerRef.current === abortController) {
                    abortControllerRef.current = null;
                    setState([false, result]);
                }
                return result;
            }
        }
        else {
            setState([false, resultPromise]);
            return resultPromise;
        }
    }), [memoizedOperation]);
    return [trigger].concat(state);
}
